<?php
	class Point {
		public $x, $y;
		public function __construct($x, $y) { $this->x = $x; $this->y = $y; }
		public function substract(Point $p) { return new Point($this->x - $p->x, $this->y - $p->y); }
		static public function relative(Point $a, Point $b) { return ($a->x * $b->y) - ($b->x * $a->y); }
		public function normalize($v = 1.0) { $x = $this->x; $y = $this->y; if ($l = $this->length) { $x /= $l; $y /= $l; } return new Point($x * $v, $y * $v); }
		public function length() { return hypot($this->x, $this->y); }
		public function __get($key) { if (method_exists($this, $key)) return $this->$key(); }
		public function mul($v) { return new static($this->x * $v, $this->y * $v); }
		static public function dot(Point $a, Point $b) { return ($a->x * $b->x) + ($a->y * $b->y); }
		static public function zero() { return new Point(0, 0); }
		static public function create($x, $y) { return new Point($x, $y); }
		static public function project(Point $v1, Point $v2) { return $v2->mul(static::dot($v1, $v2)); }
	}

	class Rectangle {
		public $left, $top, $right, $bottom;
		
		public function __construct($x1, $y1, $x2, $y2) {
			$this->left   = min($x1, $x2);
			$this->right  = max($x1, $x2);

			$this->top    = min($y1, $y2);
			$this->bottom = max($y1, $y2);
		}

		public function __get($key) {
			switch ($key) {
				case 'topLeft'    : return new Point($this->left , $this->top   );
				case 'topRight'   : return new Point($this->right, $this->top   );
				case 'bottomLeft' : return new Point($this->left , $this->bottom);
				case 'bottomRight': return new Point($this->right, $this->bottom);
			}
		}
		
		public function offsetPoint(Point $d) {
			return new Rectangle(
				$this->left   + $d->x,
				$this->top    + $d->y,
				$this->right  + $d->x,
				$this->bottom + $d->y
			);
		}
		
		static public function create($x, $y, $w, $h) {
			return new static($x, $y, $x + $w, $y + $h);
		}
	}
	
	class Line {
		public $p1, $p2;
		
		public function __construct(Point $p1, Point $p2) {
			$this->p1 = $p1;
			$this->p2 = $p2;
		}

		// Obtenemos el rectángulo que envuelve a una línea
		public function rectangle() {
			return new Rectangle($this->p1->x, $this->p1->y, $this->p2->x, $this->p2->y);
		}
		
		public function director() {
			return $this->p2->substract($this->p1);
		}
		
		static public function create($x1, $y1, $x2, $y2) {
			return new static(new Point($x1, $y1), new Point($x2, $y2));
		}
		
		public function __get($key) { if (method_exists($this, $key)) return $this->$key(); }
	}

	class Geometry {		
		static public function collisionRectangle2(Rectangle $a, Rectangle $b) {
			return !(($a->left <= $b->right) && ($a->right >= $b->left) && ($a->top <= $b->bottom) && ($a->bottom >= $b->top));
		}
		
		static public function collisionLineRectangle(Line $l, Rectangle $r) {
			// Comprueba que los rectángulos limitantes interseccionen como requisito inicial
			if (static::collisionRectangle2($r, $l->rectangle)) return false;
			
			// Obtiene el vector director de la recta
			$director = $l->director;
			
			return !static::compareSignArray(array(
				Point::relative($director, $r->topLeft    ->substract($l->p1)),
				Point::relative($director, $r->bottomRight->substract($l->p1)),
				Point::relative($director, $r->topRight   ->substract($l->p1)),
				Point::relative($director, $r->bottomLeft ->substract($l->p1))				
			));
		}	
		
		// Compara el signo de un array de números, devuelve si el signo es el mismo en todos o no.
		static public function compareSignArray($l) {
			$l = array_flip(array_map(array('Math', 'sign'), $l));
			unset($l[0]);
			return (count($l) < 2);
		}
	}

	class GeometryContainer {
		// Vector para almacenar las líneas de colisión
		public $lines = array();
		// Vector para almacenar los rectángulos de colisión
		public $rects = array();
		
		public function clear() {
			$this->lines = array();
			$this->rects = array();
		}
		
		public function add($o) {
			if ($o instanceof Line) {
				$this->lines[] = $o;
			} else if ($o instanceof Rectangle) {
				$this->rects[] = $o;
			}
		}
		
		protected function checkLinesCollision(Rectangle $r) {
			$clines = array();
			foreach ($this->lines as $l) if (Geometry::collisionLineRectangle($l, $r)) $clines[] = $l;
			return $clines;
		}
		
		// Prueba a mover un rectángulo con un vector $director-> Devuelve
		// el vector director mas apropiado para la dirección dada.
		public function moveCollision(Rectangle $r, Point $d) {
			if (!$d->length) return $d;
			
			$clinesAfter = $this->checkLinesCollision($r->offsetPoint($d));
			
			// Si no hay colisión, se devuelve el vector sin problemas.
			if (count($clinesAfter) == 0) return $d;
			
			foreach ($clinesAfter as $line) {
				$vd = Point::project($d, $line->director->normalize);
				if ( !count($this->checkLinesCollision($r->offsetPoint($vd))) ) return $vd;
			}
			
			if ( !count($this->checkLinesCollision($r->offsetPoint(Point::create($d->x,     0)))) ) return Point::create($d->x,     0);
			if ( !count($this->checkLinesCollision($r->offsetPoint(Point::create(    0, $d->y)))) ) return Point::create(    0, $d->y);

			return count($this->checkLinesCollision($r)) ? $d : Point::zero();
		}		
	}

	//class GeometryContainerOptimized extends GeometryContainer {
	class GeometryContainerOptimized {
		// Vector para almacenar las líneas de colisión
		public $lines = array(), $_lines = array();
		// Vector para almacenar los rectángulos de colisión
		public $rects = array(), $_rects = array();
		
		public $bsize;
		public $lastLinesInRect = 0;
		
		static public function hash($x, $y) { return "{$x}_{$y}"; }
		
		public function getBlocks_Line(Line $l) {
			list($bx, $by) = array($this->bsize->x, $this->bsize->y);
			$r = array();
			$director = $l->director;

			$r[] = static::hash(floor($l->p1->x / $bx), floor($l->p1->y / $by));
			$r[] = static::hash(floor($l->p2->x / $bx), floor($l->p2->y / $by));
			
			// Recorremos la línea horizontalmente
			if (abs($director->x) >= abs($director->y)) {
				if ($l->p1->x < $l->p2->x) { $p1 = $l->p1; $p2 = $l->p2; } else { $p1 = $l->p2; $p2 = $l->p1; }

				$mult = $director->y / $director->x;
				$offset = $p1->y - $mult * $p1->x;
				for ($x = floor($p1->x / $bx), $x2 = ceil($p2->x / $bx); $x <= $x2; $x++) {
					$cx = $x * $bx;
					$cy = floor(($mult * $cx + $offset) / $by);
					$r[] = static::hash($x - 1, $cy);
					$r[] = static::hash($x    , $cy);
				}
			}
			// Recorremos la línea verticalmente
			else {
				if ($l->p1->y < $l->p2->y) { $p1 = $l->p1; $p2 = $l->p2; } else { $p1 = $l->p2; $p2 = $l->p1; }

				$mult = $director->x / $director->y;
				$offset = $p1->x - $mult * $p1->y;
				for ($y = floor($p1->y / $by), $y2 = ceil($p2->y / $by); $y <= $y2; $y++) {
					$cy = $y * $by; $cx = floor(($mult * $cy + $offset) / $bx);
					$r[] = static::hash($cx, $y - 1);
					$r[] = static::hash($cx, $y    );
				}
			}
			
			return array_unique($r);
		}

		public function getBlocks_Rectangle(Rectangle $r) {
			$rr = array();
			list($bx, $by) = array($this->bsize->x, $this->bsize->y);
			list($x1, $y1) = array(floor($r->left  / $bx), floor($r->top    / $by));
			list($x2, $y2) = array(floor($r->right / $bx), floor($r->bottom / $by));
			for ($y = $y1; $y <= $y2; $y++) for ($x = $x1; $x <= $x2; $x++) $rr[] = static::hash($x, $y);
			return $rr;
		}
		
		public function __construct(Point $bsize = null) {
			if ($bsize === null) $bsize = new Point(64, 64);
			$this->bsize = $bsize;
		}
		
		public function clear() {
			$this->lines = array();
			$this->rects = array();
		}
		
		public function add($o) {
			if ($o instanceof Line) {
				foreach ($this->getBlocks_Line($o) as $p) $this->_lines[$p][] = $o;
				$this->lines[] = $o;
			} else if ($o instanceof Rectangle) {
				foreach ($this->getBlocks_Rectagle($o) as $p) $this->_rects[$p][] = $o;
				$this->rects[] = $o;
			}
		}
		
		public function getLinesInRect(Rectangle $r) {
			$lines = array();
			foreach ($this->getBlocks_Rectangle($r) as $key) if (isset($this->_lines[$key])) $lines = array_merge($lines, $this->_lines[$key]);
			return array_unique($lines);
		}
		
		protected function checkLinesCollision(Rectangle $r) {
			$clines = array();
			foreach ($this->getLinesInRect($r) as $l) if (Geometry::collisionLineRectangle($l, $r)) $clines[] = $l;
			return $clines;
		}
		
		// Prueba a mover un rectángulo con un vector $director-> Devuelve
		// el vector director mas apropiado para la dirección dada.
		public function moveCollision(Rectangle $r, Point $d) {
			if (!$d->length) return $d;
			
			$this->lastLinesInRect = count($this->getLinesInRect($r->offsetPoint($d)));
			
			$clinesAfter = $this->checkLinesCollision($r->offsetPoint($d));
			
			// Si no hay colisión, se devuelve el vector sin problemas.
			if (count($clinesAfter) == 0) return $d;
			
			foreach ($clinesAfter as $line) {
				$vd = Point::project($d, $line->director->normalize);
				if ( !count($this->checkLinesCollision($r->offsetPoint($vd))) ) return $vd;
			}
			
			if ( !count($this->checkLinesCollision($r->offsetPoint(Point::create($d->x,     0)))) ) return Point::create($d->x,     0);
			if ( !count($this->checkLinesCollision($r->offsetPoint(Point::create(    0, $d->y)))) ) return Point::create(    0, $d->y);

			return count($this->checkLinesCollision($r)) ? $d : Point::zero();
		}
	}
?>